#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <gtk/gtk.h>
#include "main.h"


/* converts a ':' separated list of items from char buffer into a new GList */
GList * get_items(char *line) {

	char *a, *b, *c ;
	GList *res ;

	if(!line || *line == '\0') return NULL ;

	res = NULL ;
	a = line ;
	
	while( (b = strchr(a,':')) ) {

		*b = '\0' ;
		while(isspace(*a) || *a == ':') a++ ;

		if(a == b + 1) continue ; 
		c = malloc(strlen(a)) ;
		strcpy(c, a) ;
		res = g_list_append(res, c) ;
		a = b + 1 ;
	}
	
	while(isspace(*a)) a++ ;
	c = malloc(strlen(a)) ;
	strcpy(c, a) ;
	res = g_list_append(res, c) ;

	return res ;
}


/* writes the new configuration file */
int write_rcfile(opt_s *o) {
	char *path, *tmp, bufor[BUFSIZ], rcfile[FILENAME_MAX] ;
	FILE *rc ;
	int i, j, l ;
	time_t date ;

	strcpy(rcfile, getenv("HOME")) ;
	strcat(rcfile, "/.gprc") ;
	rc = fopen(rcfile, "w") ;

	if( rc == NULL) {
		fprintf(stderr,
			"Configuration file (%s) cannot be opened for writing!\n", rcfile) ;
		return EXIT_FAILURE ;
	}

	date = time(NULL) ;
	fprintf(rc, "# Configuration file \n"
						  "# modified %s\n", ctime(&date)) ;

	/* saves command history */
	if(o->command_history) {
		fprintf(rc, "command_history=") ;

		l = g_list_length(o->command_history) ;
		if(l > 20) j = (l - 20) ; else j = 0 ;
		for(i = j; i < l ; i++) {
			tmp = g_list_nth_data(o->command_history, i) ;
			if(i != j) fprintf(rc, ":") ;
			fprintf(rc, "%s", tmp) ;
		}

		fprintf(rc, "\n") ;
	}

	/* saves file history */
	if(o->file_history) {
		fprintf(rc, "file_history=") ;

		l = g_list_length(o->file_history) ;
		if(l > 5) j = (l - 5) ; else j = 0 ;
		for(i = j; i < l ; i++) {
			tmp = g_list_nth_data(o->file_history, i) ;
			if(i != j) fprintf(rc, ":") ;
			fprintf(rc, "%s", tmp) ;
		}

		fprintf(rc, "\n") ;
	}

	fclose(rc) ;
	return(EXIT_SUCCESS) ;
}


/* reads the configuration file */
int read_rcfile(opt_s *o) {
	char *path, *tmp, bufor[BUFSIZ], rcfile[FILENAME_MAX] ;
	FILE *rc ;

	strcpy(rcfile, getenv("HOME")) ;
	strcat(rcfile, "/.gprc") ;

	if( !(rc = fopen(rcfile, "r")) ) {
		fprintf(stderr, "No configuration file (%s)\n", rcfile) ;
		return EXIT_FAILURE ;
	}

	while( fgets(bufor, BUFSIZ, rc) != NULL) {

		if(*bufor == '#' || isspace(*bufor)) continue ;
		if( (tmp = strchr(bufor, '\n')) ) *tmp = '\0' ;

		if(strstr(bufor, "command_history=") == bufor) {
			tmp = bufor + strlen("command_history") + 1;
			o->command_history = get_items(tmp) ;
		}
			
		if(strstr(bufor, "file_history=") == bufor) {
			tmp = bufor + strlen("file_history") + 1;
			o->file_history = get_items(tmp) ;
		}
			
	}

	fclose(rc) ;
	
	return EXIT_SUCCESS ;
}
